@@ -34,8 +34,6 @@ class PresShell;
   0x00000002  // size calculations include embellishments
 
 class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
-  friend class nsMathMLmfencedFrame;
-
  public:
   nsMathMLContainerFrame(ComputedStyle* aStyle, nsPresContext* aPresContext,
                          ClassID aID)
@@ -71,15 +69,14 @@ class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
     if (aFlags & (eLineParticipant | eSupportsContainLayoutAndPaint)) {
       return false;
     }
-    return nsContainerFrame::IsFrameOfType(
-        aFlags & ~(eMathML | eExcludesIgnorableWhitespace));
+    return nsContainerFrame::IsFrameOfType(aFlags & ~eMathML);
   }
 
-  virtual void AppendFrames(ChildListID aListID,
-                            nsFrameList& aFrameList) override;
+  void AppendFrames(ChildListID aListID, nsFrameList&& aFrameList) override;
 
-  virtual void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
-                            nsFrameList& aFrameList) override;
+  void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                    const nsLineList::iterator* aPrevFrameLine,
+                    nsFrameList&& aFrameList) override;
 
   virtual void RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override;
 
@@ -111,7 +108,7 @@ class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
   virtual void BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                 const nsDisplayListSet& aLists) override;
 
-  virtual bool ComputeCustomOverflow(nsOverflowAreas& aOverflowAreas) override;
+  bool ComputeCustomOverflow(mozilla::OverflowAreas& aOverflowAreas) override;
 
   virtual void MarkIntrinsicISizesDirty() override;
 
@@ -139,9 +136,9 @@ class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
   // helper function to apply mirroring to a horizontal coordinate, if needed.
   nscoord MirrorIfRTL(nscoord aParentWidth, nscoord aChildWidth,
                       nscoord aChildLeading) {
-    return (StyleVisibility()->mDirection
-                ? aParentWidth - aChildWidth - aChildLeading
-                : aChildLeading);
+    return StyleVisibility()->mDirection == mozilla::StyleDirection::Rtl
+               ? aParentWidth - aChildWidth - aChildLeading
+               : aChildLeading;
   }
 
   // --------------------------------------------------------------------------
@@ -207,6 +204,14 @@ class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
   nsresult TransmitAutomaticDataForMrowLikeElement();
 
  public:
+  /*
+   * Helper to render the frame as a default mrow-like container or as a visual
+   * feedback to the user when an error (typically invalid markup) was
+   * encountered during reflow. Parameters are the same as Place().
+   */
+  nsresult PlaceForError(DrawTarget* aDrawTarget, bool aPlaceOrigin,
+                         ReflowOutput& aDesiredSize);
+
   // error handlers to provide a visual feedback to the user when an error
   // (typically invalid markup) was encountered during reflow.
   nsresult ReflowError(DrawTarget* aDrawTarget, ReflowOutput& aDesiredSize);
@@ -235,9 +240,9 @@ class nsMathMLContainerFrame : public nsContainerFrame, public nsMathMLFrame {
    * Helper to call ReportToConsole when an error occurs.
    * @param aParams see nsContentUtils::ReportToConsole
    */
-  nsresult ReportErrorToConsole(const char* aErrorMsgId,
-                                const char16_t** aParams = nullptr,
-                                uint32_t aParamCount = 0);
+  nsresult ReportErrorToConsole(
+      const char* aErrorMsgId,
+      const nsTArray<nsString>& aParams = nsTArray<nsString>());
 
   // helper method to reflow a child frame. We are inline frames, and we don't
   // know our positions until reflow is finished. That's why we ask the
@@ -379,46 +384,50 @@ class nsMathMLmathBlockFrame final : public nsBlockFrame {
   // beware, mFrames is not set by nsBlockFrame
   // cannot use mFrames{.FirstChild()|.etc} since the block code doesn't set
   // mFrames
-  virtual void SetInitialChildList(ChildListID aListID,
-                                   nsFrameList& aChildList) override {
-    MOZ_ASSERT(aListID == kPrincipalList || aListID == kBackdropList,
+  void SetInitialChildList(ChildListID aListID,
+                           nsFrameList&& aChildList) override {
+    MOZ_ASSERT(aListID == mozilla::FrameChildListID::Principal ||
+                   aListID == mozilla::FrameChildListID::Backdrop,
                "unexpected frame list");
-    nsBlockFrame::SetInitialChildList(aListID, aChildList);
-    if (aListID == kPrincipalList) {
+    nsBlockFrame::SetInitialChildList(aListID, std::move(aChildList));
+    if (aListID == mozilla::FrameChildListID::Principal) {
       // re-resolve our subtree to set any mathml-expected data
       nsMathMLContainerFrame::RebuildAutomaticDataForChildren(this);
     }
   }
 
-  virtual void AppendFrames(ChildListID aListID,
-                            nsFrameList& aFrameList) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+  void AppendFrames(ChildListID aListID, nsFrameList&& aFrameList) override {
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
-    nsBlockFrame::AppendFrames(aListID, aFrameList);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    nsBlockFrame::AppendFrames(aListID, std::move(aFrameList));
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
-  virtual void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
-                            nsFrameList& aFrameList) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+  void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                    const nsLineList::iterator* aPrevFrameLine,
+                    nsFrameList&& aFrameList) override {
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
-    nsBlockFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    nsBlockFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
+                               std::move(aFrameList));
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
   virtual void RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
     nsBlockFrame::RemoveFrame(aListID, aOldFrame);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
   virtual bool IsFrameOfType(uint32_t aFlags) const override {
-    return nsBlockFrame::IsFrameOfType(
-        aFlags & ~(nsIFrame::eMathML | nsIFrame::eExcludesIgnorableWhitespace));
+    return nsBlockFrame::IsFrameOfType(aFlags & ~nsIFrame::eMathML);
   }
 
   // See nsIMathMLFrame.h
@@ -435,7 +444,7 @@ class nsMathMLmathBlockFrame final : public nsBlockFrame {
     // Bug 1301881: Do we still need to set NS_BLOCK_FLOAT_MGR?
     // AddStateBits(NS_BLOCK_FLOAT_MGR);
   }
-  virtual ~nsMathMLmathBlockFrame() {}
+  virtual ~nsMathMLmathBlockFrame() = default;
 };
 
 // --------------
@@ -449,43 +458,47 @@ class nsMathMLmathInlineFrame final : public nsInlineFrame,
   friend nsContainerFrame* NS_NewMathMLmathInlineFrame(
       mozilla::PresShell* aPresShell, ComputedStyle* aStyle);
 
-  virtual void SetInitialChildList(ChildListID aListID,
-                                   nsFrameList& aChildList) override {
-    NS_ASSERTION(aListID == kPrincipalList, "unexpected frame list");
-    nsInlineFrame::SetInitialChildList(aListID, aChildList);
+  void SetInitialChildList(ChildListID aListID,
+                           nsFrameList&& aChildList) override {
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal,
+                 "unexpected frame list");
+    nsInlineFrame::SetInitialChildList(aListID, std::move(aChildList));
     // re-resolve our subtree to set any mathml-expected data
     nsMathMLContainerFrame::RebuildAutomaticDataForChildren(this);
   }
 
-  virtual void AppendFrames(ChildListID aListID,
-                            nsFrameList& aFrameList) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+  void AppendFrames(ChildListID aListID, nsFrameList&& aFrameList) override {
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
-    nsInlineFrame::AppendFrames(aListID, aFrameList);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    nsInlineFrame::AppendFrames(aListID, std::move(aFrameList));
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
-  virtual void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
-                            nsFrameList& aFrameList) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+  void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                    const nsLineList::iterator* aPrevFrameLine,
+                    nsFrameList&& aFrameList) override {
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
-    nsInlineFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    nsInlineFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
+                                std::move(aFrameList));
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
   virtual void RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override {
-    NS_ASSERTION(aListID == kPrincipalList || aListID == kNoReflowPrincipalList,
+    NS_ASSERTION(aListID == mozilla::FrameChildListID::Principal ||
+                     aListID == mozilla::FrameChildListID::NoReflowPrincipal,
                  "unexpected frame list");
     nsInlineFrame::RemoveFrame(aListID, aOldFrame);
-    if (MOZ_LIKELY(aListID == kPrincipalList))
+    if (MOZ_LIKELY(aListID == mozilla::FrameChildListID::Principal))
       nsMathMLContainerFrame::ReLayoutChildren(this);
   }
 
   virtual bool IsFrameOfType(uint32_t aFlags) const override {
-    return nsInlineFrame::IsFrameOfType(
-        aFlags & ~(nsIFrame::eMathML | nsIFrame::eExcludesIgnorableWhitespace));
+    return nsInlineFrame::IsFrameOfType(aFlags & ~nsIFrame::eMathML);
   }
 
   bool IsMrowLike() override {
@@ -497,7 +510,7 @@ class nsMathMLmathInlineFrame final : public nsInlineFrame,
                                    nsPresContext* aPresContext)
       : nsInlineFrame(aStyle, aPresContext, kClassID) {}
 
-  virtual ~nsMathMLmathInlineFrame() {}
+  virtual ~nsMathMLmathInlineFrame() = default;
 };
 
 #endif /* nsMathMLContainerFrame_h___ */
