@@ -6,6 +6,7 @@
 
 #include "mozilla/ComputedStyle.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_layout.h"
 #include "nsTableColFrame.h"
 #include "nsTableFrame.h"
 #include "nsStyleConsts.h"
@@ -68,17 +69,23 @@ nsresult nsTableColGroupFrame::AddColsToTable(int32_t aFirstColIndex,
 
   // set the col indices of the col frames and and add col info to the table
   int32_t colIndex = aFirstColIndex;
-  nsFrameList::Enumerator e(aCols);
-  for (; !e.AtEnd(); e.Next()) {
-    ((nsTableColFrame*)e.get())->SetColIndex(colIndex);
+
+  // XXX: We cannot use range-based for loop because InsertCol() can destroy the
+  // nsTableColFrame in the slice we're traversing! Need to check the validity
+  // of *colIter.
+  auto colIter = aCols.begin();
+  for (auto colIterEnd = aCols.end(); *colIter && colIter != colIterEnd;
+       ++colIter) {
+    auto* colFrame = static_cast<nsTableColFrame*>(*colIter);
+    colFrame->SetColIndex(colIndex);
     mColCount++;
-    tableFrame->InsertCol((nsTableColFrame&)*e.get(), colIndex);
+    tableFrame->InsertCol(*colFrame, colIndex);
     colIndex++;
   }
 
-  for (nsFrameList::Enumerator eTail = e.GetUnlimitedEnumerator();
-       !eTail.AtEnd(); eTail.Next()) {
-    ((nsTableColFrame*)eTail.get())->SetColIndex(colIndex);
+  for (; *colIter; ++colIter) {
+    auto* colFrame = static_cast<nsTableColFrame*>(*colIter);
+    colFrame->SetColIndex(colIndex);
     colIndex++;
   }
 
@@ -95,7 +102,7 @@ nsresult nsTableColGroupFrame::AddColsToTable(int32_t aFirstColIndex,
 
 nsTableColGroupFrame* nsTableColGroupFrame::GetLastRealColGroup(
     nsTableFrame* aTableFrame) {
-  nsFrameList colGroups = aTableFrame->GetColGroups();
+  const nsFrameList& colGroups = aTableFrame->GetColGroups();
 
   auto lastColGroup = static_cast<nsTableColGroupFrame*>(colGroups.LastChild());
   if (!lastColGroup) {
@@ -111,10 +118,10 @@ nsTableColGroupFrame* nsTableColGroupFrame::GetLastRealColGroup(
 
 // don't set mColCount here, it is done in AddColsToTable
 void nsTableColGroupFrame::SetInitialChildList(ChildListID aListID,
-                                               nsFrameList& aChildList) {
+                                               nsFrameList&& aChildList) {
   MOZ_ASSERT(mFrames.IsEmpty(),
              "unexpected second call to SetInitialChildList");
-  MOZ_ASSERT(aListID == kPrincipalList, "unexpected child list");
+  MOZ_ASSERT(aListID == FrameChildListID::Principal, "unexpected child list");
 #ifdef DEBUG
   for (nsIFrame* f : aChildList) {
     MOZ_ASSERT(f->GetParent() == this, "Unexpected parent");
@@ -126,7 +133,7 @@ void nsTableColGroupFrame::SetInitialChildList(ChildListID aListID,
     return;
   }
 
-  mFrames.AppendFrames(this, aChildList);
+  mFrames.AppendFrames(this, std::move(aChildList));
 }
 
 /* virtual */
@@ -149,8 +156,8 @@ void nsTableColGroupFrame::DidSetComputedStyle(
 }
 
 void nsTableColGroupFrame::AppendFrames(ChildListID aListID,
-                                        nsFrameList& aFrameList) {
-  NS_ASSERTION(aListID == kPrincipalList, "unexpected child list");
+                                        nsFrameList&& aFrameList) {
+  NS_ASSERTION(aListID == FrameChildListID::Principal, "unexpected child list");
 
   nsTableColFrame* col = GetFirstColumn();
   nsTableColFrame* nextCol;
@@ -160,7 +167,7 @@ void nsTableColGroupFrame::AppendFrames(ChildListID aListID,
     // since the HTML spec says to ignore the span of a colgroup if it
     // has content columns in it.
     nextCol = col->GetNextCol();
-    RemoveFrame(kPrincipalList, col);
+    RemoveFrame(FrameChildListID::Principal, col);
     col = nextCol;
   }
 
@@ -171,14 +178,15 @@ void nsTableColGroupFrame::AppendFrames(ChildListID aListID,
   MOZ_ASSERT(!col || col->GetColType() == eColContent,
              "What's going on with our columns?");
 
-  const nsFrameList::Slice& newFrames = mFrames.AppendFrames(this, aFrameList);
+  const nsFrameList::Slice& newFrames =
+      mFrames.AppendFrames(this, std::move(aFrameList));
   InsertColsReflow(GetStartColumnIndex() + mColCount, newFrames);
 }
 
 void nsTableColGroupFrame::InsertFrames(
     ChildListID aListID, nsIFrame* aPrevFrame,
-    const nsLineList::iterator* aPrevFrameLine, nsFrameList& aFrameList) {
-  NS_ASSERTION(aListID == kPrincipalList, "unexpected child list");
+    const nsLineList::iterator* aPrevFrameLine, nsFrameList&& aFrameList) {
+  NS_ASSERTION(aListID == FrameChildListID::Principal, "unexpected child list");
   NS_ASSERTION(!aPrevFrame || aPrevFrame->GetParent() == this,
                "inserting after sibling frame with different parent");
 
@@ -197,7 +205,7 @@ void nsTableColGroupFrame::InsertFrames(
       // We'll want to insert at the beginning
       aPrevFrame = nullptr;
     }
-    RemoveFrame(kPrincipalList, col);
+    RemoveFrame(FrameChildListID::Principal, col);
     col = nextCol;
   }
 
@@ -215,7 +223,7 @@ void nsTableColGroupFrame::InsertFrames(
                "Shouldn't be inserting before a spanned colframe");
 
   const nsFrameList::Slice& newFrames =
-      mFrames.InsertFrames(this, aPrevFrame, aFrameList);
+      mFrames.InsertFrames(this, aPrevFrame, std::move(aFrameList));
   nsIFrame* prevFrame = nsTableFrame::GetFrameAtOrBefore(
       this, aPrevFrame, LayoutFrameType::TableCol);
 
@@ -229,7 +237,7 @@ void nsTableColGroupFrame::InsertColsReflow(int32_t aColIndex,
                                             const nsFrameList::Slice& aCols) {
   AddColsToTable(aColIndex, true, aCols);
 
-  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::TreeChange,
+  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::FrameAndAncestors,
                                 NS_FRAME_HAS_DIRTY_CHILDREN);
 }
 
@@ -253,13 +261,13 @@ void nsTableColGroupFrame::RemoveChild(nsTableColFrame& aChild,
     }
   }
 
-  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::TreeChange,
+  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::FrameAndAncestors,
                                 NS_FRAME_HAS_DIRTY_CHILDREN);
 }
 
 void nsTableColGroupFrame::RemoveFrame(ChildListID aListID,
                                        nsIFrame* aOldFrame) {
-  NS_ASSERTION(aListID == kPrincipalList, "unexpected child list");
+  NS_ASSERTION(aListID == FrameChildListID::Principal, "unexpected child list");
 
   if (!aOldFrame) {
     return;
@@ -277,7 +285,7 @@ void nsTableColGroupFrame::RemoveFrame(ChildListID aListID,
 #ifdef DEBUG
 #endif
         nextCol = col->GetNextCol();
-        RemoveFrame(kPrincipalList, col);
+        RemoveFrame(FrameChildListID::Principal, col);
         col = nextCol;
       }
     }
@@ -297,18 +305,17 @@ void nsTableColGroupFrame::RemoveFrame(ChildListID aListID,
   }
 }
 
-nsIFrame::LogicalSides nsTableColGroupFrame::GetLogicalSkipSides(
-    const ReflowInput* aReflowInput) const {
+nsIFrame::LogicalSides nsTableColGroupFrame::GetLogicalSkipSides() const {
   LogicalSides skip(mWritingMode);
   if (MOZ_UNLIKELY(StyleBorder()->mBoxDecorationBreak ==
                    StyleBoxDecorationBreak::Clone)) {
     return skip;
   }
 
-  if (nullptr != GetPrevInFlow()) {
+  if (GetPrevInFlow()) {
     skip |= eLogicalSideBitsBStart;
   }
-  if (nullptr != GetNextInFlow()) {
+  if (GetNextInFlow()) {
     skip |= eLogicalSideBitsBEnd;
   }
   return skip;
@@ -348,7 +355,6 @@ void nsTableColGroupFrame::Reflow(nsPresContext* aPresContext,
   }
 
   aDesiredSize.ClearSize();
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 void nsTableColGroupFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
@@ -424,8 +430,9 @@ void nsTableColGroupFrame::InvalidateFrame(uint32_t aDisplayItemKey,
                                            bool aRebuildDisplayItems) {
   nsIFrame::InvalidateFrame(aDisplayItemKey, aRebuildDisplayItems);
   if (GetTableFrame()->IsBorderCollapse()) {
-    GetParent()->InvalidateFrameWithRect(
-        GetVisualOverflowRect() + GetPosition(), aDisplayItemKey, false);
+    const bool rebuild = StaticPrefs::layout_display_list_retain_sc();
+    GetParent()->InvalidateFrameWithRect(InkOverflowRect() + GetPosition(),
+                                         aDisplayItemKey, rebuild);
   }
 }
 
@@ -438,12 +445,12 @@ void nsTableColGroupFrame::InvalidateFrameWithRect(const nsRect& aRect,
   // we get an inactive layer created and this is computed
   // within FrameLayerBuilder
   GetParent()->InvalidateFrameWithRect(aRect + GetPosition(), aDisplayItemKey,
-                                       false);
+                                       aRebuildDisplayItems);
 }
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsTableColGroupFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("TableColGroup"), aResult);
+  return MakeFrameName(u"TableColGroup"_ns, aResult);
 }
 
 void nsTableColGroupFrame::Dump(int32_t aIndent) {
@@ -461,7 +468,7 @@ void nsTableColGroupFrame::Dump(int32_t aIndent) {
       IsSynthetic() ? "true" : "false");
 
   // verify the colindices
-  int32_t j = GetStartColumnIndex();
+  DebugOnly<int32_t> j = GetStartColumnIndex();
   nsTableColFrame* col = GetFirstColumn();
   while (col) {
     NS_ASSERTION(j == col->GetColIndex(), "wrong colindex on col frame");
