@@ -8,13 +8,24 @@
 // leaking to window scope.
 {
   const MozPopupElement = MozElements.MozElementMixin(XULPopupElement);
-  MozElements.MozAutocompleteRichlistboxPopup = class MozAutocompleteRichlistboxPopup extends MozPopupElement {
+  MozElements.MozAutocompleteRichlistboxPopup = class MozAutocompleteRichlistboxPopup extends (
+    MozPopupElement
+  ) {
     constructor() {
       super();
 
+      this.attachShadow({ mode: "open" });
+
+      {
+        let slot = document.createElement("slot");
+        slot.part = "content";
+        this.shadowRoot.appendChild(slot);
+      }
+
       this.mInput = null;
       this.mPopupOpen = false;
       this._currentIndex = 0;
+      this._disabledItemClicked = false;
 
       this.setListeners();
     }
@@ -59,16 +70,22 @@
             }
 
             switch (event.type) {
+              case "mousedown":
+                this._disabledItemClicked =
+                  !!event.target.closest("richlistitem")?.disabled;
+                break;
               case "mouseup":
                 // Don't call onPopupClick for the scrollbar buttons, thumb,
                 // slider, etc. If we hit the richlistbox and not a
                 // richlistitem, we ignore the event.
                 if (
                   event.target.closest("richlistbox,richlistitem").localName ==
-                  "richlistitem"
+                    "richlistitem" &&
+                  !this._disabledItemClicked
                 ) {
                   this.onPopupClick(event);
                 }
+                this._disabledItemClicked = false;
                 break;
               case "mousemove":
                 if (Date.now() - this.mLastMoveTime <= 30) {
@@ -97,6 +114,7 @@
           },
         };
       }
+      this.richlistbox.addEventListener("mousedown", this.listEvents);
       this.richlistbox.addEventListener("mouseup", this.listEvents);
       this.richlistbox.addEventListener("mousemove", this.listEvents);
     }
@@ -110,7 +128,7 @@
 
     static get markup() {
       return `
-      <richlistbox class="autocomplete-richlistbox" flex="1"/>
+      <richlistbox class="autocomplete-richlistbox"/>
     `;
     }
 
@@ -150,7 +168,6 @@
           this.richlistbox.selectedItem || this.richlistbox.firstElementChild
         );
       }
-      return val;
     }
 
     get selectedIndex() {
@@ -176,9 +193,7 @@
       return Number(this.getAttribute("overflowpadding"));
     }
 
-    set view(val) {
-      return val;
-    }
+    set view(val) {}
 
     get view() {
       return this.mInput.controller;
@@ -187,7 +202,7 @@
     closePopup() {
       if (this.mPopupOpen) {
         this.hidePopup();
-        this.removeAttribute("width");
+        this.style.removeProperty("--panel-width");
       }
     }
 
@@ -252,7 +267,7 @@
         this.selectedIndex = -1;
 
         var width = aElement.getBoundingClientRect().width;
-        this.setAttribute("width", width > 100 ? width : 100);
+        this.style.setProperty("--panel-width", Math.max(width, 100) + "px");
         // invalidate() depends on the width attribute
         this._invalidate();
 
@@ -280,7 +295,7 @@
       }
 
       if (this.mPopupOpen) {
-        delete this._adjustHeightOnPopupShown;
+        this._adjustHeightOnPopupShown = false;
         this._adjustHeightRAFToken = requestAnimationFrame(() =>
           this.adjustHeight()
         );
@@ -337,16 +352,11 @@
         height = lastRowRect.bottom - firstRowRect.top + this._rlbPadding;
       }
 
-      let currentHeight = this.richlistbox.getBoundingClientRect().height;
-      if (height <= currentHeight) {
-        this._collapseUnusedItems();
-      }
-      this.richlistbox.style.removeProperty("height");
-      // We need to get the ceiling of the calculated value to ensure that the box fully contains
-      // all of its contents and doesn't cause a scrollbar since nsIBoxObject only expects a
-      // `long`. e.g. if `height` is 99.5 the richlistbox would render at height 99px with a
-      // scrollbar for the extra 0.5px.
-      this.richlistbox.height = Math.ceil(height);
+      this._collapseUnusedItems();
+
+      // We need to get the ceiling of the calculated value to ensure that the
+      // box fully contains all of its contents and doesn't cause a scrollbar.
+      this.richlistbox.style.height = Math.ceil(height) + "px";
     }
 
     _appendCurrentResult(invalidateReason) {
@@ -394,6 +404,8 @@
             "autofill-clear-button",
             "autofill-insecureWarning",
             "generatedPassword",
+            "generic",
+            "importableLearnMore",
             "importableLogins",
             "insecureWarning",
             "loginsFooter",
@@ -425,6 +437,14 @@
             case "autofill-insecureWarning":
               options = { is: "autocomplete-creditcard-insecure-field" };
               break;
+            case "generic":
+              options = { is: "autocomplete-generic-richlistitem" };
+              break;
+            case "importableLearnMore":
+              options = {
+                is: "autocomplete-importable-learn-more-richlistitem",
+              };
+              break;
             case "importableLogins":
               options = { is: "autocomplete-importable-logins-richlistitem" };
               break;
@@ -535,6 +555,7 @@
 
     disconnectedCallback() {
       if (this.listEvents) {
+        this.richlistbox.removeEventListener("mousedown", this.listEvents);
         this.richlistbox.removeEventListener("mouseup", this.listEvents);
         this.richlistbox.removeEventListener("mousemove", this.listEvents);
         delete this.listEvents;
@@ -556,7 +577,7 @@
 
       this.addEventListener("popupshown", event => {
         if (this._adjustHeightOnPopupShown) {
-          delete this._adjustHeightOnPopupShown;
+          this._adjustHeightOnPopupShown = false;
           this.adjustHeight();
         }
       });
