@@ -10,7 +10,14 @@ loadTestSubscript("head_disco.js");
 // latest AMO API, by replacing API_RESPONSE_FILE's content with latest AMO API
 // response, e.g. from https://addons.allizom.org/api/v4/discovery/?lang=en-US
 // The response must contain at least one theme, and one extension.
-const API_RESPONSE_FILE = RELATIVE_DIR + "discovery/api_response.json";
+
+const API_RESPONSE_FILE = PathUtils.join(
+  Services.dirsvc.get("CurWorkD", Ci.nsIFile).path,
+  // Trim empty component from splitting with trailing slash.
+  ...RELATIVE_DIR.split("/").filter(c => c.length),
+  "discovery",
+  "api_response.json"
+);
 
 const AMO_TEST_HOST = "rewritten-for-testing.addons.allizom.org";
 
@@ -199,7 +206,7 @@ async function testAddonUninstall(card) {
   );
 }
 
-add_task(async function setup() {
+add_setup(async function () {
   await SpecialPowers.pushPrefEnv({
     set: [
       [
@@ -348,14 +355,12 @@ add_task(async function install_from_discopane() {
   await promiseDiscopaneUpdate(win);
   await waitForAllImagesLoaded(win);
 
-  Services.telemetry.clearEvents();
-
   // Test extension install.
   let installExtensionPromise = promiseAddonInstall(amoServer, {
     manifest: {
       name: "My Awesome Add-on",
       description: "Test extension install button",
-      applications: { gecko: { id: FIRST_EXTENSION_ID } },
+      browser_specific_settings: { gecko: { id: FIRST_EXTENSION_ID } },
       permissions: ["<all_urls>"],
     },
   });
@@ -367,7 +372,7 @@ add_task(async function install_from_discopane() {
     manifest: {
       name: "My Fancy Theme",
       description: "Test theme install button",
-      applications: { gecko: { id: FIRST_THEME_ID } },
+      browser_specific_settings: { gecko: { id: FIRST_THEME_ID } },
       theme: {
         colors: {
           tab_selected: "red",
@@ -394,33 +399,6 @@ add_task(async function install_from_discopane() {
     "The Install buttons should be replaced with Manage buttons"
   );
 
-  assertAboutAddonsTelemetryEvents([
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      null,
-      {
-        action: "installFromRecommendation",
-        view: "discover",
-        addonId: FIRST_EXTENSION_ID,
-        type: "extension",
-      },
-    ],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      null,
-      {
-        action: "installFromRecommendation",
-        view: "discover",
-        addonId: FIRST_THEME_ID,
-        type: "theme",
-      },
-    ],
-  ]);
-
   // End of the testing installation from a card.
 
   // Click on the Manage button to verify that it does something useful,
@@ -439,24 +417,6 @@ add_task(async function install_from_discopane() {
     // TODO bug 1540253: Check that the "recommended" badge is visible.
   }
 
-  assertAboutAddonsTelemetryEvents(
-    [
-      [
-        "addonsManager",
-        "action",
-        "aboutAddons",
-        null,
-        {
-          action: "manage",
-          view: "discover",
-          addonId: FIRST_EXTENSION_ID,
-          type: "extension",
-        },
-      ],
-    ],
-    { methods: ["action"] }
-  );
-
   // Now we are going to force an updated rendering and check that the cards are
   // in the expected order, and then test uninstallation of the above add-ons.
   await switchToDiscoView(win);
@@ -622,80 +582,6 @@ add_task(async function discopane_no_cookies() {
   await closeView(win);
 });
 
-// Telemetry for interaction that have not been covered by other tests yet.
-// - "Find more addons" button.
-// - Link to add-on listing in recommended add-on card.
-// Other interaction is already covered elsewhere:
-// - Install/Manage buttons have bee tested before in install_from_discopane.
-// - "Learn more" button is checked by browser_html_discover_view_clientid.js.
-add_task(async function discopane_interaction_telemetry() {
-  await SpecialPowers.pushPrefEnv({
-    set: [["extensions.getAddons.link.url", `http://${AMO_TEST_HOST}/dummy`]],
-  });
-  // Minimal API response to get the link in recommended-addon-card to render.
-  const DUMMY_EXTENSION_ID = "dummy@extensionid";
-  const apiResponse = {
-    results: [
-      {
-        addon: {
-          guid: DUMMY_EXTENSION_ID,
-          type: "extension",
-          authors: [
-            {
-              name: "Some author",
-            },
-          ],
-          url: `http://${AMO_TEST_HOST}/dummy`,
-          icon_url: `http://${AMO_TEST_HOST}/png`,
-        },
-      },
-    ],
-  };
-  let apiHandler = new DiscoveryAPIHandler(JSON.stringify(apiResponse));
-
-  let expectedAmoUrlFor = where => {
-    // eslint-disable-next-line max-len
-    return `http://${AMO_TEST_HOST}/dummy?utm_source=firefox-browser&utm_medium=firefox-browser&utm_content=${where}`;
-  };
-
-  let testClickInDiscoCard = async (selector, utmContentParam) => {
-    let tabbrowser = win.windowRoot.ownerGlobal.gBrowser;
-    let tabPromise = BrowserTestUtils.waitForNewTab(tabbrowser);
-    getDiscoveryElement(win)
-      .querySelector(selector)
-      .click();
-    let tab = await tabPromise;
-    is(
-      tab.linkedBrowser.currentURI.spec,
-      expectedAmoUrlFor(utmContentParam),
-      "Expected URL of new tab"
-    );
-    BrowserTestUtils.removeTab(tab);
-  };
-
-  let win = await loadInitialView("discover");
-  await promiseDiscopaneUpdate(win);
-  is(await waitForAllImagesLoaded(win), 1, "One recommendation in results");
-
-  Services.telemetry.clearEvents();
-
-  // "Find more add-ons" button.
-  await testClickInDiscoCard("[action='open-amo']", "find-more-link-bottom");
-
-  // Link to AMO listing
-  await testClickInDiscoCard(".disco-addon-author a", "discopane-entry-link");
-
-  assertAboutAddonsTelemetryEvents([
-    ["addonsManager", "link", "aboutAddons", "discomore", { view: "discover" }],
-    ["addonsManager", "link", "aboutAddons", "discohome", { view: "discover" }],
-  ]);
-
-  is(apiHandler.requestCount, 1, "Discovery API should be fetched once");
-
-  await closeView(win);
-  await SpecialPowers.popPrefEnv();
-});
-
 // The CSP of about:addons whitelists http:, but not data:, hence we are
 // loading a little red data: image which gets blocked by the CSP.
 add_task(async function csp_img_src() {
