@@ -21,7 +21,6 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 
 static void WriteMsg(const NS_tchar* path, const char* status) {
@@ -142,6 +141,7 @@ int NS_main(int argc, NS_tchar** argv) {
         "   or: setup-symlink dir1 dir2 file symlink\n"
         "   or: remove-symlink dir1 dir2 file symlink\n"
         "   or: check-symlink symlink\n"
+        "   or: check-umask existing-umask\n"
         "   or: post-update\n"
         "   or: create-update-dir\n"
         "\n"
@@ -286,6 +286,37 @@ int NS_main(int argc, NS_tchar** argv) {
 #endif
   }
 
+  if (!NS_tstrcmp(argv[1], NS_T("check-umask"))) {
+#ifdef XP_UNIX
+    // Discover the current value of the umask.  There is no way to read the
+    // umask without changing it.  The system call is specified as unable to
+    // fail.
+    uint32_t umask = ::umask(0777);
+    ::umask(umask);
+
+    NS_tchar logFilePath[MAXPATHLEN];
+    if (!NS_tvsnprintf(logFilePath,
+                       sizeof(logFilePath) / sizeof(logFilePath[0]), NS_T("%s"),
+                       argv[2])) {
+      return 1;
+    }
+
+    FILE* logFP = NS_tfopen(logFilePath, NS_T("wb"));
+    if (!logFP) {
+      return 1;
+    }
+    fprintf(logFP, "check-umask\numask-%d\n", umask);
+
+    fclose(logFP);
+    logFP = nullptr;
+
+    return 0;
+#else
+    // Not implemented on non-Unix platforms
+    return 1;
+#endif
+  }
+
   if (!NS_tstrcmp(argv[1], NS_T("wait-for-service-stop"))) {
 #if defined(XP_WIN) && defined(MOZ_MAINTENANCE_SERVICE)
     const int maxWaitSeconds = NS_ttoi(argv[3]);
@@ -344,8 +375,7 @@ int NS_main(int argc, NS_tchar** argv) {
   if (!NS_tstrcmp(argv[1], NS_T("create-update-dir"))) {
 #ifdef XP_WIN
     mozilla::UniquePtr<wchar_t[]> updateDir;
-    HRESULT result = GetCommonUpdateDirectory(
-        argv[2], SetPermissionsOf::BaseDirIfNotExists, updateDir);
+    HRESULT result = GetCommonUpdateDirectory(argv[2], updateDir);
     return SUCCEEDED(result) ? 0 : 1;
 #else
     // Not implemented on non-Windows platforms
